/*
Author: Adrian Kuegel
Date: August 25th, 2011
Algorithm: Preflow Push with gap heuristic
*/

#include <stdio.h>
#include <assert.h>
#include <queue>
#include <limits.h>
#include <vector>
#include <algorithm>
#include <string.h>
using namespace std;

typedef vector<int> VI;
typedef queue<int> QI;
typedef pair<int, int> PII;
typedef vector<PII> VPII;

#define MAXV 10002

VPII a[MAXV];
VI g[MAXV+1]; // for gap heuristic
int h[MAXV];
int e[MAXV];
int pos[MAXV];
VPII::iterator current[MAXV];
int ecap[60000];
int edges;

int Preflow(int n, int src, int sink)
{
	QI q;
	memset(e, 0, sizeof(int) * n);
	memset(h, 0, sizeof(int) * n);
	g[n].clear();
	for (int i=0; i<n; ++i) {
		current[i] = a[i].begin();
		if (i != src && i != sink) {
			pos[i] = g[0].size();
			g[0].push_back(i);
		}
	}
	h[src] = n;
	for (VPII::iterator it=a[src].begin(); it!=a[src].end(); ++it)
		if (ecap[it->second] > 0)
		{
			e[it->first] = ecap[it->second];
			e[src] -= ecap[it->second];
			ecap[(it->second + edges) % (2 * edges)] += ecap[it->second];
			ecap[it->second] = 0;
			if (it->first != sink)
				q.push(it->first);
		}
	int max_not_empty = 0;
	// for the gap heuristic, we maintain following invariant:
	// g[i] is non-empty for i in [0...max_not_empty]
	// g[i] is empty for i in [max_not_empty + 1...n]
	while (!q.empty())
	{
		int u = q.front();
		q.pop();
		while (e[u] > 0)
			if (current[u] == a[u].end())
			{
				// if we are here, it means we skipped through the whole list since the last update of h[u]
				// so there is no v with h[v] + 1 = h[u] and c[u][v] - f[u][v] > 0
				// since we maintain the property h[v] >= h[u]-1 this means in fact h[v] >= h[u]
				int lowest = INT_MAX; //2*n + 1;
				// relabel
				for (VPII::iterator it = a[u].begin(); it != a[u].end(); ++it)
					if (ecap[it->second] > 0)
						lowest = min(lowest, h[it->first]);
				++lowest;
				// now lowest > min_v (h[v]) >= h[u]
				int oldh = h[u];
				h[u] = lowest;
				// gap heuristic
				if (oldh <= n) {
					// maintain data structures (in O(1))
					// remove vertex u from height oldh list
					VI::iterator it = g[oldh].begin() + pos[u];
					swap(*it, g[oldh].back());
					pos[*it] = pos[u];
					g[oldh].pop_back();
					// insert it in height lowest list
					if (lowest <= n) {
						// we know the node v for which h[v] = lowest-1 is still in list g[lowest-1]
						// so g[lowest-1] is not empty -> lowest-1 <= max_not_empty -> lowest <= max_not_empty+1
						if (g[lowest].empty()) {
							// this means lowest >= max_not_empty+1, so it must follow:
							// lowest == max_not_empty + 1
							++max_not_empty;
						}
						pos[u] = g[lowest].size();
						g[lowest].push_back(u);
					}
					if (oldh && g[oldh].empty()) {
						// each element is moved at most once, therefore overall O(n)
						for (int i=oldh+1; i<=max_not_empty; ++i) {
							for (VI::iterator it = g[i].begin(); it!=g[i].end(); ++it) {
								h[*it] = n + 1;
								current[*it] = a[*it].begin();
							}
							g[i].clear();
						}
						// oldh-1 must be 0 or not empty, otherwise we would have emptied g[oldh] already
						max_not_empty = oldh-1;
					}
				}
				current[u] = a[u].begin();
			}
			else
			{
				int v = current[u]->first;
				if (ecap[current[u]->second] > 0 && h[u] == h[v] + 1)
				{
					if (e[v] == 0 && v != src && v != sink)
						q.push(v);
					int d = min(e[u], ecap[current[u]->second]);
					ecap[current[u]->second] -= d;
					ecap[(current[u]->second+edges)%(2*edges)] += d;
					e[u] -= d;
					e[v] += d;
				}
				else
					++current[u];
			}
	}
	return e[sink];
}

char grid[100][101];

int main() {
	int tc, n, m, d, f, b;
	scanf("%d", &tc);
	while(tc--) {
		scanf("%d %d", &m, &n);
		scanf("%d %d %d", &d, &f, &b);
		for (int i=0; i<n; ++i)
			scanf("%s", grid[i]);
		int cost = 0;
		for (int i=0; i<n; ++i) {
			if (grid[i][0] == '.') {
				grid[i][0] = '#';
				cost += f;
			}
			if (grid[i][m-1] == '.') {
				grid[i][m-1] = '#';
				cost += f;
			}
		}
		for (int i=1; i+1<m; ++i) {
			if (grid[0][i] == '.') {
				grid[0][i] = '#';
				cost += f;
			}
			if (grid[n-1][i] == '.') {
				grid[n-1][i] = '#';
				cost += f;
			}
		}
		int nodes = 2 + n * m;
		for (int i=0; i<nodes; ++i) {
			a[i].clear();
			g[i].clear();
		}
		edges = n * m * 3;
		int l = 0;
		bool possible = true;
		for (int i=0; i<n; ++i) {
			for (int j=0; j<m; ++j) {
				// add edges to right and bottom neighbor
				if (i+1 < n) {
					ecap[l] = ecap[l + edges] = b;
					a[1+i*m+j].push_back(PII(1+(i+1)*m+j, l));
					a[1+(i+1)*m+j].push_back(PII(1+i*m+j, l+edges));
					++l;
				}
				if (j+1 < m) {
					ecap[l] = ecap[l + edges] = b;
					a[1+i*m+j].push_back(PII(1+i*m+j+1, l));
					a[1+i*m+j+1].push_back(PII(1+i*m+j, l+edges));
					++l;
				}
				if (grid[i][j] == '#') {
					int cap = d;
					if (!i || i+1 == n || !j || j+1 == m)
						cap = b*4;
					ecap[l] = cap;
					ecap[l + edges] = 0;
					a[0].push_back(PII(1+i*m+j, l));
					a[1+i*m+j].push_back(PII(0, l + edges));
					++l;
				}
				else {
					ecap[l] = f;
					ecap[l + edges] = 0;
					a[1+i*m+j].push_back(PII(nodes-1, l));
					a[nodes-1].push_back(PII(1+i*m+j, l+edges));
					++l;
				}
			}
		}
		cost += Preflow(nodes, 0, nodes-1);
		printf("%d\n", cost);
	}
	return 0;
}
